package de.jottyfan.minecraft.quickiefabric.blockentity;

import java.util.ArrayList;
import java.util.List;

import net.minecraft.block.BlockState;
import net.minecraft.block.entity.LootableContainerBlockEntity;
import net.minecraft.entity.Entity;
import net.minecraft.entity.ItemEntity;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.inventory.Inventories;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.Items;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.screen.ScreenHandler;
import net.minecraft.text.LiteralText;
import net.minecraft.text.Style;
import net.minecraft.text.Text;
import net.minecraft.text.TranslatableText;
import net.minecraft.util.Formatting;
import net.minecraft.util.Tickable;
import net.minecraft.util.collection.DefaultedList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Box;
import net.minecraft.world.World;

/**
 * 
 * @author jotty
 *
 */
public class ItemHoarderBlockEntity extends LootableContainerBlockEntity implements Tickable {
	private DefaultedList<ItemStack> stacks;
	private float suckradius;

	public ItemHoarderBlockEntity() {
		super(QuickieFabricBlockEntity.ITEMHOARDER);
		stacks = DefaultedList.ofSize(54, ItemStack.EMPTY);
		suckradius = 4f; // TODO: make it level up - able and start with 2
	}

	// TODO: see https://fabricmc.net/wiki/tutorial:containers for a real chest

	private final boolean setStackToSlots(ItemStack stack) {
		for (ItemStack slot : stacks) {
			if (slot.getItem().equals(stack.getItem())) {
				slot.increment(stack.getCount());
				return true;
			}
		} // if not found, seek for an empty stack instead
		for (ItemStack slot : stacks) {
			if (slot.isEmpty()) {
				Integer index = stacks.indexOf(slot);
				stacks.set(index, stack.copy());
				return true;
			}
		}
		return false;
	}

	@Override
	public void tick() {
		BlockPos pos = getPos();
		World world = getWorld();
		Box box = new Box(pos).expand(suckradius);
		List<Entity> entities = world.getOtherEntities(null, box);
		for (Entity entity : entities) {
			if (entity instanceof ItemEntity) {
				ItemEntity itemEntity = (ItemEntity) entity;
				if (itemEntity.isAlive()) {
					ItemStack stack = itemEntity.getStack();
					if (stack != null) {
						if (setStackToSlots(stack)) {
							itemEntity.remove();
						} // else inventory is full
					}
				}
			}
		}
	}

	@Override
	public CompoundTag toTag(CompoundTag tag) {
		super.toTag(tag);
		if (!this.serializeLootTable(tag)) {
			Inventories.toTag(tag, this.stacks);
		}
		return tag;
	}

	@Override
	public void fromTag(BlockState state, CompoundTag tag) {
		super.fromTag(state, tag);
		this.stacks = DefaultedList.ofSize(this.size(), ItemStack.EMPTY);
		if (!this.deserializeLootTable(tag)) {
			Inventories.fromTag(tag, this.stacks);
		}
	}

	public List<ItemStack> getStacks() {
		return stacks;
	}

	@Override
	public int size() {
		return 54; // container chest size (9 * 6)
	}

	@Override
	protected DefaultedList<ItemStack> getInvStackList() {
		return stacks;
	}

	@Override
	protected void setInvStackList(DefaultedList<ItemStack> list) {
		this.stacks = list;
	}

	@Override
	protected ScreenHandler createScreenHandler(int syncId, PlayerInventory playerInventory) {
		// TODO: implement, see https://fabricmc.net/wiki/tutorial:containers
		return null;
	}

	@Override
	protected Text getContainerName() {
		return new TranslatableText("container.itemhoarder");
	}

	/**
	 * get a summary of content for the chat box
	 * 
	 * @return the summary
	 */
	public List<Text> getSummary() {
		List<Text> list = new ArrayList<>();
		for (ItemStack stack : stacks) {
			Item item = stack.getItem();
			if (item != Items.AIR) {
				Integer amount = stack.getCount();
				Text text = new LiteralText(String.format("%dx ", amount))
						.append(new TranslatableText(stack.getTranslationKey())).setStyle(Style.EMPTY.withColor(Formatting.GOLD));
				list.add(text);
			}
		}
		return list;
	}
}
