package de.jottyfan.minecraft.quickiefabric.container;

import de.jottyfan.minecraft.quickiefabric.items.ItemBackpack;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.inventory.SimpleInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.ListTag;
import net.minecraft.sound.SoundCategory;
import net.minecraft.sound.SoundEvents;
import net.minecraft.util.Hand;

/**
 * 
 * @author jotty
 *
 */
public class BackpackInventory extends SimpleInventory {
	private Hand hand;

	private BackpackInventory(CompoundTag tag, BackpackScreenHandler handler) {
		super(ItemBackpack.SLOTSIZE);
		readItemsFromTag(super.size(), tag);
	}

	public static final BackpackInventory getInventory(BackpackScreenHandler handler, PlayerEntity player,
			ItemStack stack) {
		return new BackpackInventory(init(stack).getTag().getCompound("backpack"), handler);
	}
	
	public BackpackInventory(ItemStack stack) {
		this(init(stack).getTag().getCompound("backpack"), null);
	}
	
	private final static ItemStack init(ItemStack stack) {
		if (stack != null) {
			if (!stack.hasTag()) {
				stack.setTag(new CompoundTag());
			}
			if (!stack.getTag().contains("backpack")) {
				stack.getTag().put("backpack", new CompoundTag());
			}
		}
		return stack;
	}

	@Override
	public void onOpen(PlayerEntity player) {
		super.onOpen(player);
		player.playSound(SoundEvents.BLOCK_WOOL_PLACE, SoundCategory.PLAYERS, 1f, 1f);
	}

	@Override
	public void onClose(PlayerEntity player) {
		super.onClose(player);
		ItemStack stack = player.getStackInHand(hand);
		if (stack != null) {
			if (!stack.hasTag()) {
				stack.setTag(new CompoundTag());
			}
			stack.getTag().put("backpack", writeItemsToTag());
		}
		player.getStackInHand(hand).setTag(stack.getTag());
		player.playSound(SoundEvents.BLOCK_WOOL_PLACE, SoundCategory.PLAYERS, 1f, 1f);
	}

	private void readItemsFromTag(Integer size, CompoundTag tag) {
		ListTag listTag = tag.getList("items", 10);
		for (int i = 0; i < listTag.size(); ++i) {
			CompoundTag compoundTag = listTag.getCompound(i);
			int slot = compoundTag.getInt("slot");

			if (slot >= 0 && slot < size) {
				super.setStack(slot, ItemStack.fromTag(compoundTag));
			}
		}
	}

	private CompoundTag writeItemsToTag() {
		ListTag listTag = new ListTag();
		for (int i = 0; i < super.size(); ++i) {
			ItemStack itemStack = (ItemStack) super.getStack(i);
			if (!(itemStack == null) && !itemStack.isEmpty()) {
				CompoundTag compoundTag = new CompoundTag();
				compoundTag.putInt("slot", i);
				compoundTag = itemStack.toTag(compoundTag);
				listTag.add(compoundTag);
			}
		}
		CompoundTag tag = new CompoundTag();
		tag.put("items", listTag);
		return tag;
	}
	
	public void setHand(Hand hand) {
		this.hand = hand;
	}
}
