package de.jottyfan.minecraft.quickiefabric.init;

import java.util.Arrays;
import java.util.List;
import java.util.function.Supplier;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.jottyfan.minecraft.quickiefabric.blockentity.DrillBlockDownEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.DrillBlockEastEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.DrillBlockNorthEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.DrillBlockSouthEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.DrillBlockWestEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.EmptyLavaHoarderBlockEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.ItemHoarderBlockEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.MonsterHoarderBlockEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.QuickieFabricBlockEntity;
import de.jottyfan.minecraft.quickiefabric.blocks.QuickieBlocks;
import de.jottyfan.minecraft.quickiefabric.container.BackpackScreenHandler;
import de.jottyfan.minecraft.quickiefabric.event.BreakBlockCallback;
import de.jottyfan.minecraft.quickiefabric.event.EventBlockBreak;
import de.jottyfan.minecraft.quickiefabric.items.QuickieItems;
import de.jottyfan.minecraft.quickiefabric.tools.QuickieTools;
import net.fabricmc.fabric.api.client.itemgroup.FabricItemGroupBuilder;
import net.fabricmc.fabric.api.loot.v1.FabricLootPoolBuilder;
import net.fabricmc.fabric.api.loot.v1.event.LootTableLoadingCallback;
import net.fabricmc.fabric.api.registry.FuelRegistry;
import net.fabricmc.fabric.api.screenhandler.v1.ScreenHandlerRegistry;
import net.minecraft.block.Block;
import net.minecraft.block.Blocks;
import net.minecraft.block.ComposterBlock;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.block.entity.BlockEntityType;
import net.minecraft.block.entity.BlockEntityType.Builder;
import net.minecraft.item.BlockItem;
import net.minecraft.item.Item;
import net.minecraft.item.ItemGroup;
import net.minecraft.item.ItemStack;
import net.minecraft.loot.ConstantLootTableRange;
import net.minecraft.loot.condition.RandomChanceLootCondition;
import net.minecraft.loot.entry.ItemEntry;
import net.minecraft.screen.ScreenHandlerType;
import net.minecraft.structure.rule.BlockMatchRuleTest;
import net.minecraft.util.ActionResult;
import net.minecraft.util.Identifier;
import net.minecraft.util.registry.BuiltinRegistries;
import net.minecraft.util.registry.Registry;
import net.minecraft.world.gen.decorator.Decorator;
import net.minecraft.world.gen.decorator.RangeDecoratorConfig;
import net.minecraft.world.gen.feature.ConfiguredFeature;
import net.minecraft.world.gen.feature.Feature;
import net.minecraft.world.gen.feature.OreFeatureConfig;

/**
 * 
 * @author jotty
 *
 */
public class RegistryManager {
	private static final Logger LOGGER = LogManager.getLogger(RegistryManager.class);

	public static final String QUICKIEFABRIC = "quickiefabric";

	public static final Identifier BACKPACK_IDENTIFIER = new Identifier(QUICKIEFABRIC, "backpack");
	public static final ScreenHandlerType<BackpackScreenHandler> BACKPACK_SCREEN_HANDLER = ScreenHandlerRegistry
			.registerExtended(RegistryManager.BACKPACK_IDENTIFIER, BackpackScreenHandler::new);

	public static final ConfiguredFeature<?, ?> FEATURE_ORENETHERSULPHOR = Feature.ORE
			.configure(new OreFeatureConfig(OreFeatureConfig.Rules.BASE_STONE_NETHER,
					QuickieBlocks.ORE_NETHER_SULPHOR.getDefaultState(), 24))
			.decorate(Decorator.RANGE.configure(new RangeDecoratorConfig(0, 0, 128)).spreadHorizontally().repeat(10));

	public static final ConfiguredFeature<?, ?> FEATURE_ORESALPETER = Feature.ORE
			.configure(new OreFeatureConfig(OreFeatureConfig.Rules.BASE_STONE_OVERWORLD,
					QuickieBlocks.ORE_SALPETER.getDefaultState(), 12))
			.decorate(Decorator.RANGE.configure(new RangeDecoratorConfig(0, 0, 128))).spreadHorizontally().repeat(10);

	public static final ConfiguredFeature<?, ?> FEATURE_ORESULPHOR = Feature.ORE
			.configure(new OreFeatureConfig(OreFeatureConfig.Rules.BASE_STONE_OVERWORLD,
					QuickieBlocks.ORE_SULPHOR.getDefaultState(), 16))
			.decorate(Decorator.RANGE.configure(new RangeDecoratorConfig(4, 32, 255))).spreadHorizontally().repeat(4);

	public static final ConfiguredFeature<?, ?> FEATURE_DIRTSALPETER = Feature.ORE
			.configure(
					new OreFeatureConfig(new BlockMatchRuleTest(Blocks.DIRT), QuickieBlocks.DIRT_SALPETER.getDefaultState(), 3))
			.decorate(Decorator.RANGE.configure(new RangeDecoratorConfig(4, 0, 255))).spreadHorizontally().repeatRandomly(4);

	public static final ConfiguredFeature<?, ?> FEATURE_SANDSALPETER = Feature.ORE
			.configure(
					new OreFeatureConfig(new BlockMatchRuleTest(Blocks.SAND), QuickieBlocks.SAND_SALPETER.getDefaultState(), 3))
			.decorate(Decorator.RANGE.configure(new RangeDecoratorConfig(4, 0, 255))).spreadHorizontally().repeatRandomly(4);

	public static final ConfiguredFeature<?, ?> FEATURE_ORESANDSALPETER = Feature.ORE
			.configure(new OreFeatureConfig(new BlockMatchRuleTest(Blocks.SANDSTONE),
					QuickieBlocks.ORE_SAND_SALPETER.getDefaultState(), 3))
			.decorate(Decorator.RANGE.configure(new RangeDecoratorConfig(4, 0, 255))).spreadHorizontally().repeatRandomly(4);

	public static final List<ConfiguredFeature<?, ?>> FEATURE_UNDERGROUND_ORES = Arrays.asList(FEATURE_ORESALPETER,
			FEATURE_ORESULPHOR, FEATURE_DIRTSALPETER, FEATURE_SANDSALPETER, FEATURE_ORESANDSALPETER);
	public static final List<ConfiguredFeature<?, ?>> FEATURE_UNDERGROUND_DECORATIONS = Arrays
			.asList(FEATURE_ORENETHERSULPHOR);

	public static final ItemGroup QUICKIEFABRIC_GROUP = FabricItemGroupBuilder
			.create(new Identifier(QUICKIEFABRIC, "all")).icon(() -> new ItemStack(QuickieItems.SPEEDPOWDER))
			.appendItems(stacks -> {
				stacks.add(new ItemStack(QuickieItems.SALPETER));
				stacks.add(new ItemStack(QuickieItems.SULPHOR));
				stacks.add(new ItemStack(QuickieItems.SPEEDPOWDER));
				stacks.add(new ItemStack(QuickieItems.LEVELUP));
				stacks.add(new ItemStack(QuickieItems.PENCIL));
				stacks.add(new ItemStack(QuickieItems.ROTTEN_FLESH_STRIPES));
				stacks.add(new ItemStack(QuickieItems.CARROTSTACK));
				stacks.add(new ItemStack(QuickieItems.COTTON));
				stacks.add(new ItemStack(QuickieItems.COTTONSEED));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_BROWN));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_WHITE));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_BLACK));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_BLUE));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_CYAN));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_GREEN));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_PINK));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_RED));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_YELLOW));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_DARKGRAY));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_LIGHTGRAY));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_LIGHTGREEN));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_MAGENTA));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_ORANGE));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_PURPLE));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_LIGHTBLUE));
				stacks.add(new ItemStack(QuickieTools.SPEEDPOWDERAXE));
				stacks.add(new ItemStack(QuickieTools.SPEEDPOWDERPICKAXE));
				stacks.add(new ItemStack(QuickieTools.SPEEDPOWDERSHOVEL));
				stacks.add(new ItemStack(QuickieBlocks.DIRT_SALPETER));
				stacks.add(new ItemStack(QuickieBlocks.ORE_NETHER_SULPHOR));
				stacks.add(new ItemStack(QuickieBlocks.ORE_SALPETER));
				stacks.add(new ItemStack(QuickieBlocks.ORE_SAND_SALPETER));
				stacks.add(new ItemStack(QuickieBlocks.ORE_SULPHOR));
				stacks.add(new ItemStack(QuickieBlocks.SAND_SALPETER));
				stacks.add(new ItemStack(QuickieBlocks.LAVAHOARDER));
				stacks.add(new ItemStack(QuickieBlocks.EMPTYLAVAHOARDER));
				stacks.add(new ItemStack(QuickieBlocks.ITEMHOARDER));
				stacks.add(new ItemStack(QuickieBlocks.MONSTERHOARDER));
				stacks.add(new ItemStack(QuickieBlocks.KELPSTACK));
				stacks.add(new ItemStack(QuickieBlocks.BLOCKSULPHOR));
				stacks.add(new ItemStack(QuickieBlocks.DRILL_DOWN));
				stacks.add(new ItemStack(QuickieBlocks.DRILL_EAST));
				stacks.add(new ItemStack(QuickieBlocks.DRILL_SOUTH));
				stacks.add(new ItemStack(QuickieBlocks.DRILL_WEST));
				stacks.add(new ItemStack(QuickieBlocks.DRILL_NORTH));
				stacks.add(new ItemStack(QuickieBlocks.DRILLSTOP));
			}).build();

	private static final void registerBlock(Block block, String name) {
		Registry.register(Registry.BLOCK, new Identifier(QUICKIEFABRIC, name), block);
		Registry.register(Registry.ITEM, new Identifier(QUICKIEFABRIC, name),
				new BlockItem(block, new Item.Settings().group(RegistryManager.QUICKIEFABRIC_GROUP)));
	}

	private static final void registerItem(Item item, String name) {
		Registry.register(Registry.ITEM, new Identifier(QUICKIEFABRIC, name), item);
	}

	public static final <T extends BlockEntity> BlockEntityType<? extends T> registerBlockEntity(String name,
			Supplier<? extends T> supplier, Block... blocks) {
		String fullname = new StringBuilder().append(QUICKIEFABRIC).append(":").append(name).toString();
		Builder<? extends T> builder = BlockEntityType.Builder.create(supplier, blocks);
		BlockEntityType<? extends T> blockEntityType = builder.build(null);
		return Registry.register(Registry.BLOCK_ENTITY_TYPE, fullname, blockEntityType);
	}

	@SuppressWarnings("unchecked")
	public static final void registerBlockEntities() {
		LOGGER.debug("registering quickiefabric block entities");
		QuickieFabricBlockEntity.ITEMHOARDER = (BlockEntityType<ItemHoarderBlockEntity>) registerBlockEntity(
				"itemhoarderblockentity", ItemHoarderBlockEntity::new, QuickieBlocks.ITEMHOARDER);
		QuickieFabricBlockEntity.MONSTERHOARDER = (BlockEntityType<MonsterHoarderBlockEntity>) registerBlockEntity(
				"monsterhoarderblockentity", MonsterHoarderBlockEntity::new, QuickieBlocks.MONSTERHOARDER);
		QuickieFabricBlockEntity.EMPTYLAVAHOARDER = (BlockEntityType<EmptyLavaHoarderBlockEntity>) registerBlockEntity(
				"emptylavahoarderblockentity", EmptyLavaHoarderBlockEntity::new, QuickieBlocks.EMPTYLAVAHOARDER,
				QuickieBlocks.LAVAHOARDER);
		QuickieFabricBlockEntity.DRILL_DOWN = (BlockEntityType<DrillBlockDownEntity>) registerBlockEntity("drillblockdownentity",
				DrillBlockDownEntity::new, QuickieBlocks.DRILL_DOWN);
		QuickieFabricBlockEntity.DRILL_EAST = (BlockEntityType<DrillBlockEastEntity>) registerBlockEntity("drillblockeastentity",
				DrillBlockEastEntity::new, QuickieBlocks.DRILL_EAST);
		QuickieFabricBlockEntity.DRILL_SOUTH = (BlockEntityType<DrillBlockSouthEntity>) registerBlockEntity("drillblocksouthentity",
				DrillBlockSouthEntity::new, QuickieBlocks.DRILL_SOUTH);
		QuickieFabricBlockEntity.DRILL_WEST = (BlockEntityType<DrillBlockWestEntity>) registerBlockEntity("drillblockwestentity",
				DrillBlockWestEntity::new, QuickieBlocks.DRILL_WEST);
		QuickieFabricBlockEntity.DRILL_NORTH = (BlockEntityType<DrillBlockNorthEntity>) registerBlockEntity("drillblocknorthentity",
				DrillBlockNorthEntity::new, QuickieBlocks.DRILL_NORTH);
	}

	public static final void registerBlocks() {
		LOGGER.debug("registering quickiefabric blocks");
		registerBlock(QuickieBlocks.DIRT_SALPETER, "dirtsalpeter");
		registerBlock(QuickieBlocks.ORE_NETHER_SULPHOR, "orenethersulphor");
		registerBlock(QuickieBlocks.ORE_SALPETER, "oresalpeter");
		registerBlock(QuickieBlocks.ORE_SAND_SALPETER, "oresandsalpeter");
		registerBlock(QuickieBlocks.ORE_SULPHOR, "oresulphor");
		registerBlock(QuickieBlocks.SAND_SALPETER, "sandsalpeter");
		registerBlock(QuickieBlocks.LAVAHOARDER, "lavahoarder");
		registerBlock(QuickieBlocks.EMPTYLAVAHOARDER, "emptylavahoarder");
		registerBlock(QuickieBlocks.ITEMHOARDER, "itemhoarder");
		registerBlock(QuickieBlocks.MONSTERHOARDER, "monsterhoarder");
		registerBlock(QuickieBlocks.KELPSTACK, "kelpstack");
		registerBlock(QuickieBlocks.COTTONPLANT, "cottonplant");
		registerBlock(QuickieBlocks.BLOCKSULPHOR, "blocksulphor");
		registerBlock(QuickieBlocks.DRILL_DOWN, "drill");
		registerBlock(QuickieBlocks.DRILL_EAST, "drilleast");
		registerBlock(QuickieBlocks.DRILL_SOUTH, "drillsouth");
		registerBlock(QuickieBlocks.DRILL_WEST, "drillwest");
		registerBlock(QuickieBlocks.DRILL_NORTH, "drillnorth");
		registerBlock(QuickieBlocks.DRILLSTOP, "drillstop");
	}

	public static final void registerItems() {
		LOGGER.debug("registering quickiefabric items");
		registerItem(QuickieItems.SPEEDPOWDER, "speedpowder");
		registerItem(QuickieItems.LEVELUP, "levelup");
		registerItem(QuickieItems.PENCIL, "pencil");
		registerItem(QuickieItems.SALPETER, "salpeter");
		registerItem(QuickieItems.SULPHOR, "sulphor");
		registerItem(QuickieItems.ROTTEN_FLESH_STRIPES, "rotten_flesh_stripes");
		registerItem(QuickieItems.CARROTSTACK, "carrotstack");
		registerItem(QuickieItems.COTTON, "cotton");
		registerItem(QuickieItems.COTTONSEED, "cottonseed");
		registerItem(QuickieItems.BACKPACK_BROWN, "backpack_brown");
		registerItem(QuickieItems.BACKPACK_WHITE, "backpack_white");
		registerItem(QuickieItems.BACKPACK_BLACK, "backpack_black");
		registerItem(QuickieItems.BACKPACK_BLUE, "backpack_blue");
		registerItem(QuickieItems.BACKPACK_CYAN, "backpack_cyan");
		registerItem(QuickieItems.BACKPACK_GREEN, "backpack_green");
		registerItem(QuickieItems.BACKPACK_PINK, "backpack_pink");
		registerItem(QuickieItems.BACKPACK_RED, "backpack_red");
		registerItem(QuickieItems.BACKPACK_YELLOW, "backpack_yellow");
		registerItem(QuickieItems.BACKPACK_DARKGRAY, "backpack_darkgray");
		registerItem(QuickieItems.BACKPACK_LIGHTGRAY, "backpack_lightgray");
		registerItem(QuickieItems.BACKPACK_LIGHTGREEN, "backpack_lightgreen");
		registerItem(QuickieItems.BACKPACK_MAGENTA, "backpack_magenta");
		registerItem(QuickieItems.BACKPACK_ORANGE, "backpack_orange");
		registerItem(QuickieItems.BACKPACK_PURPLE, "backpack_purple");
		registerItem(QuickieItems.BACKPACK_LIGHTBLUE, "backpack_lightblue");

		ComposterBlock.ITEM_TO_LEVEL_INCREASE_CHANCE.put(QuickieItems.COTTONSEED, 0.5f);
		ComposterBlock.ITEM_TO_LEVEL_INCREASE_CHANCE.put(QuickieItems.COTTON, 0.75f);

		FuelRegistry.INSTANCE.add(QuickieItems.SULPHOR, 200);
		FuelRegistry.INSTANCE.add(QuickieBlocks.BLOCKSULPHOR, 2000);
	}

	public static final void registerTools() {
		LOGGER.debug("registering quickiefabric tools");
		registerItem(QuickieTools.SPEEDPOWDERAXE, "speedpowderaxe");
		registerItem(QuickieTools.SPEEDPOWDERPICKAXE, "speedpowderpickaxe");
		registerItem(QuickieTools.SPEEDPOWDERSHOVEL, "speedpowdershovel");
	}

	public static final void registerContainer() {
	}

	public static final void registerEvents() {
		LOGGER.debug("registering quickiefabric events");
		BreakBlockCallback.EVENT.register((world, blockPos, blockState, playerEntity) -> {
			new EventBlockBreak().doBreakBlock(world, blockPos, blockState, playerEntity);
			return ActionResult.SUCCESS;
		});
	}

	/**
	 * register ore generation feature
	 * 
	 * @param name    the name for the identifier
	 * @param feature the feature
	 */
	private static final void registerFeature(String name, ConfiguredFeature<?, ?> feature) {
		Registry.register(BuiltinRegistries.CONFIGURED_FEATURE, new Identifier(QUICKIEFABRIC, name), feature);
	}

	/**
	 * register ore generation features
	 */
	public static final void registerFeatures() {
		registerFeature("feature_orenethersulphor", FEATURE_ORENETHERSULPHOR);
		registerFeature("feature_oresalpeter", FEATURE_ORESALPETER);
		registerFeature("feature_oresulphor", FEATURE_ORESULPHOR);
		registerFeature("feature_dirtsalpeter", FEATURE_DIRTSALPETER);
		registerFeature("feature_sandsalpeter", FEATURE_SANDSALPETER);
		registerFeature("feature_oresandsalpeter", FEATURE_ORESANDSALPETER);
	}

	public static final void registerLootings() {
		LootTableLoadingCallback.EVENT.register((resourceManager, lootManager, id, supplier, setter) -> {
			if (isGrass(id)) {
				FabricLootPoolBuilder poolBuilder = FabricLootPoolBuilder.builder().rolls(ConstantLootTableRange.create(1))
						.withCondition(RandomChanceLootCondition.builder(0.125f).build())
						.withEntry(ItemEntry.builder(QuickieItems.COTTONSEED).build());
				supplier.withPool(poolBuilder.build());
			}
		});
	}

	private final static boolean isGrass(Identifier id) {
		return id.equals(new Identifier("blocks/grass")) || id.equals(new Identifier("blocks/fern"))
				|| id.equals(new Identifier("blocks/tall_grass"));
	}

}
