package de.jottyfan.minecraft.quickiefabric.items;

import de.jottyfan.minecraft.quickiefabric.container.BackpackInventory;
import de.jottyfan.minecraft.quickiefabric.container.BackpackScreenHandler;
import de.jottyfan.minecraft.quickiefabric.init.RegistryManager;
import io.netty.buffer.Unpooled;
import net.fabricmc.fabric.api.screenhandler.v1.ExtendedScreenHandlerFactory;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.Blocks;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.block.entity.BlockEntityType;
import net.minecraft.block.entity.ChestBlockEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.ItemUsageContext;
import net.minecraft.network.PacketByteBuf;
import net.minecraft.screen.ScreenHandler;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.LiteralText;
import net.minecraft.text.Text;
import net.minecraft.text.TranslatableText;
import net.minecraft.util.ActionResult;
import net.minecraft.util.Hand;
import net.minecraft.util.TypedActionResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

/**
 * 
 * @author jotty
 *
 */
public class ItemBackpack extends Item {
	public final static Integer SLOTSIZE = 54;

	public ItemBackpack() {
		super(new Item.Settings().group(RegistryManager.QUICKIEFABRIC_GROUP).maxCount(1));
	}

	@Override
	public TypedActionResult<ItemStack> use(World world, PlayerEntity player, Hand hand) {
		ItemStack itemStack = player.getStackInHand(hand);
		if (!world.isClient) {
			player.openHandledScreen(new ExtendedScreenHandlerFactory() {
				@Override
				public void writeScreenOpeningData(ServerPlayerEntity serverPlayerEntity, PacketByteBuf packetByteBuf) {
					packetByteBuf.writeItemStack(itemStack);
					packetByteBuf.writeByte(hand == Hand.MAIN_HAND ? 1 : 0);
				}

				@Override
				public Text getDisplayName() {
					return new LiteralText(RegistryManager.BACKPACK_IDENTIFIER.toString());
				}

				@Override
				public ScreenHandler createMenu(int syncId, PlayerInventory inv, PlayerEntity player) {
					PacketByteBuf buf = new PacketByteBuf(Unpooled.buffer());
					buf.writeItemStack(itemStack);
					buf.writeByte(hand == Hand.MAIN_HAND ? 1 : 0);
					return new BackpackScreenHandler(syncId, inv, buf);
				}
			});
		}
		return new TypedActionResult<ItemStack>(ActionResult.SUCCESS, itemStack);
	}

	@Override
	public ActionResult useOnBlock(ItemUsageContext context) {
		BlockPos pos = context.getBlockPos();
		World world = context.getWorld();
		PlayerEntity player = context.getPlayer();
		BlockState blockState = world.getBlockState(pos);
		Block block = blockState.getBlock();
		if (Blocks.CHEST.equals(block)) {
			BlockEntity entity = world.getBlockEntity(pos);
			BlockEntityType<?> type = entity.getType();
			if (BlockEntityType.CHEST.equals(type)) {
				ChestBlockEntity cbe = (ChestBlockEntity) entity;
				BackpackInventory bi = new BackpackInventory(context.getStack());
				bi.setHand(context.getHand());
				if (cbe.isEmpty()) {
					for (int slot = 0; slot < ItemBackpack.SLOTSIZE; slot++) {
						if (slot < cbe.size()) {
							cbe.setStack(slot, bi.getStack(slot));
							bi.removeStack(slot);
						}
					}
					bi.onClose(player);
					if (!world.isClient) {
						player.sendMessage(new TranslatableText("msg.backpack.transfer.cleared"), false);
					}
				} else if (bi.isEmpty()) {
					for (int slot = 0; slot < ItemBackpack.SLOTSIZE; slot++) {
						if (slot < cbe.size()) {
							bi.setStack(slot, cbe.getStack(slot));
							cbe.setStack(slot, ItemStack.EMPTY);
						}
					}
					bi.onClose(player);
					if (!world.isClient) {
						player.sendMessage(new TranslatableText("msg.backpack.transfer.filled"), false);
					}
				} else {
					if (!world.isClient) {
						player.sendMessage(new TranslatableText("msg.backpack.transfer.cancel"), false);
					}
				}
				// alternative:
				// TODO: create a new dialog for transferring items from backpack to chest and
				// back
				// with the help of some buttons (such as all or so)
			}
		}
		return super.useOnBlock(context);
	}
}
