package de.jottyfan.minecraft.quickiefabric.tools;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

import de.jottyfan.minecraft.quickiefabric.tools.externalmods.Byg;
import de.jottyfan.minecraft.quickiefabric.tools.externalmods.IdentifierGroup;
import de.jottyfan.minecraft.quickiefabric.tools.externalmods.Terrestria;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.util.Identifier;

/**
 * 
 * @author jotty
 *
 */
public interface ToolRangeable {
	public static final Set<Block> SHOVEL_EFFECTIVE_ON = Sets
			.newHashSet(new Block[] { Blocks.GRAVEL, Blocks.SAND, Blocks.GRASS_BLOCK, Blocks.DIRT, Blocks.CLAY,
					Blocks.FARMLAND, Blocks.GRASS_PATH, Blocks.RED_SAND, Blocks.SOUL_SAND });

	public static final Set<Identifier> SHOVEL_EXTERNAL_EFFECTIVE_ON = mergeSets(
			Byg.createFromStrings("black_sand", "white_sand", "peat", "meadow_dirt", "mud_block", "blue_sand", "purple_sand"),
			Terrestria.createFromStrings("basalt_sand"));

	public static final List<List<Block>> AXE_EFFECTIVE_ON = Lists.newArrayList(
			Arrays.asList(Blocks.ACACIA_LOG, Blocks.STRIPPED_ACACIA_LOG, Blocks.ACACIA_WOOD, Blocks.STRIPPED_ACACIA_WOOD),
			Arrays.asList(Blocks.BIRCH_LOG, Blocks.STRIPPED_BIRCH_LOG, Blocks.BIRCH_WOOD, Blocks.STRIPPED_BIRCH_WOOD),
			Arrays.asList(Blocks.DARK_OAK_LOG, Blocks.STRIPPED_DARK_OAK_LOG, Blocks.DARK_OAK_WOOD,
					Blocks.STRIPPED_DARK_OAK_WOOD),
			Arrays.asList(Blocks.JUNGLE_LOG, Blocks.STRIPPED_JUNGLE_LOG, Blocks.JUNGLE_WOOD, Blocks.STRIPPED_JUNGLE_WOOD),
			Arrays.asList(Blocks.OAK_LOG, Blocks.STRIPPED_OAK_LOG, Blocks.OAK_WOOD, Blocks.STRIPPED_OAK_WOOD),
			Arrays.asList(Blocks.SPRUCE_LOG, Blocks.STRIPPED_SPRUCE_LOG, Blocks.SPRUCE_WOOD, Blocks.STRIPPED_SPRUCE_WOOD),
			Arrays.asList(Blocks.ACACIA_PLANKS), Arrays.asList(Blocks.BIRCH_PLANKS), Arrays.asList(Blocks.DARK_OAK_PLANKS),
			Arrays.asList(Blocks.JUNGLE_PLANKS), Arrays.asList(Blocks.OAK_PLANKS), Arrays.asList(Blocks.SPRUCE_PLANKS),
			Arrays.asList(Blocks.ACACIA_SLAB), Arrays.asList(Blocks.BIRCH_SLAB), Arrays.asList(Blocks.DARK_OAK_SLAB),
			Arrays.asList(Blocks.JUNGLE_SLAB), Arrays.asList(Blocks.OAK_SLAB), Arrays.asList(Blocks.SPRUCE_SLAB),
			Arrays.asList(Blocks.ACACIA_STAIRS), Arrays.asList(Blocks.BIRCH_STAIRS), Arrays.asList(Blocks.DARK_OAK_STAIRS),
			Arrays.asList(Blocks.JUNGLE_STAIRS), Arrays.asList(Blocks.OAK_STAIRS), Arrays.asList(Blocks.SPRUCE_STAIRS),
			Arrays.asList(Blocks.POTTED_BROWN_MUSHROOM, Blocks.BROWN_MUSHROOM_BLOCK, Blocks.BROWN_MUSHROOM,
					Blocks.MUSHROOM_STEM),
			Arrays.asList(Blocks.POTTED_RED_MUSHROOM, Blocks.RED_MUSHROOM_BLOCK, Blocks.RED_MUSHROOM, Blocks.MUSHROOM_STEM));

	public static final Set<IdentifierGroup> AXE_EFFECTIVE_ON_EXTERNAL = mergeGroupSets(
			Terrestria.createFromSet(Terrestria.WOOD.getGroups("log", "quarter_log", "log_bare", "log_top", "wood", "")),
			// only because terrestria leaves are not instanceof LeavesBlock
			Terrestria.createFromSet(Terrestria.WOOD.getGroups("leaves")),
			Byg.createFromSet(Byg.WOOD.getGroups("log", "wood")),
			Byg.createFromSet(Byg.MUSHROOM.getGroups("block", "stem")));

	public static final Set<Block> PICKAXE_EFFECTIVE_ON = Sets.newHashSet(new Block[] { Blocks.GLOWSTONE });

	/**
	 * merge all sets
	 * 
	 * @param sets the sets
	 * @return one merged set
	 */
	@SafeVarargs
	public static Set<Identifier> mergeSets(Set<Identifier>... sets) {
		Set<Identifier> result = new HashSet<>();
		for (Set<Identifier> set : sets) {
			result.addAll(set);
		}
		return result;
	}

	/**
	 * merge all group sets
	 * 
	 * @param sets the sets
	 * @return one merged set
	 */
	@SafeVarargs
	public static Set<IdentifierGroup> mergeGroupSets(Set<IdentifierGroup>... sets) {
		Set<IdentifierGroup> result = new HashSet<>();
		for (Set<IdentifierGroup> set : sets) {
			result.addAll(set);
		}
		return result;
	}

	/**
	 * @param stack the item stack that keeps the range
	 * @return range of blocks to be harvested
	 */
	public HarvestRange getRange(ItemStack stack);

	/**
	 * check if this block state is one that affects the neighbor blocks to break
	 * also if they are from the same type
	 * 
	 * @param blockState the block state of the current block
	 * @return true or false
	 */
	public boolean canBreakNeighbors(BlockState blockState);

	/**
	 * get list of blocks that belong together
	 * 
	 * @param block of the set
	 * @return the list of blocks or null if not found
	 */
	public List<Block> getBlockList(Block block);
}
