package de.jottyfan.minecraft.quickiefabric.tools;

import java.util.List;

import com.google.common.collect.Lists;

import de.jottyfan.minecraft.quickiefabric.tools.externalmods.IdentifierGroup;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.LeavesBlock;
import net.minecraft.item.AxeItem;
import net.minecraft.item.ItemStack;
import net.minecraft.item.ToolMaterial;
import net.minecraft.util.Identifier;
import net.minecraft.util.registry.Registry;

/**
 * 
 * @author jotty
 *
 */
public class ToolRangeableAxe extends AxeItem implements ToolRangeable {

	protected ToolRangeableAxe(ToolMaterial material, float attachDamage, float attackSpeedIn, Settings settings) {
		super(material, attachDamage, attackSpeedIn, settings);
	}

	@Override
	public HarvestRange getRange(ItemStack stack) {
		// TODO: get the range from the stack
		return new HarvestRange(64, 128, 64); // trees bigger than that are too heavy for one small axe...
	}

	@Override
	public float getMiningSpeedMultiplier(ItemStack stack, BlockState state) {
		return getBlockList(state.getBlock()) != null ? this.miningSpeed : super.getMiningSpeedMultiplier(stack, state);
	}

	@Override
	public boolean canBreakNeighbors(BlockState blockIn) {
		return getBlockList(blockIn.getBlock()) != null;
	}

	@Override
	public List<Block> getBlockList(Block block) {
		for (List<Block> blockList : AXE_EFFECTIVE_ON) {
			if (blockList.contains(block)) {
				return blockList;
			}
		}
		for (IdentifierGroup externalIDs : AXE_EFFECTIVE_ON_EXTERNAL) {
			for (Identifier externalID : externalIDs.getIdentifiers()) {
				Block registeredBlock = Registry.BLOCK.get(externalID); // may be null if mods are not available
				if (registeredBlock != null && registeredBlock.equals(block)) {
					return Lists.newArrayList(block);
				}
			}
		}
		if (block instanceof LeavesBlock) {
			return Lists.newArrayList(block);
		}
		return null;
	}
}
